import argparse
import os

import pandas as pd

import classifications


def restructure_dataframe(original_df, folder_name, suffix):
    columns = original_df.columns[:2]
    final_df = pd.DataFrame()

    # Rename columns based on folder_name
    final_df["tree_bonsai_activitytype"] = (
        original_df["tree_bonsai"] if folder_name == "activitytype" else None
    )
    final_df["tree_bonsai_flowobject"] = (
        original_df["tree_bonsai"] if folder_name == "flowobject" else None
    )

    for col in columns:
        if folder_name == "activitytype":
            final_df["tree_other_activitytype"] = original_df[col]
            final_df["tree_other_flowobject"] = None  # Ensure flowobject column is None
        elif folder_name == "flowobject":
            final_df[
                "tree_other_activitytype"
            ] = None  # Ensure activitytype column is None
            final_df["tree_other_flowobject"] = original_df[col]

    final_df["comment"] = original_df["comment"]
    final_df["prefixed_id"] = original_df["prefixed_id"]
    final_df["skos_uri"] = original_df["skos_uri"]

    final_df["other_classification"] = suffix.replace("tree_", "")

    # Replace NaN with None for clarity
    final_df = final_df.where(pd.notnull(final_df), None)

    return final_df


def main():
    parser = argparse.ArgumentParser(
        description="Merge all Bonsai concordance tables into one"
    )
    parser.add_argument(
        "--output_dir",
        type=str,
        default="output",
        help="Directory to save the updated DataFrames",
    )

    args = parser.parse_args()

    # (table, other_classification_name)
    attributes = dir(classifications.activitytype.datapackage)
    # value = getattr(getattr(classifications, 'activitytype.datapackage'), attr_string)
    conc_activitytype = [
        (
            getattr(
                getattr(classifications, "activitytype"), "datapackage"
            ).__getattribute__(attr),
            attr[len("conc_bonsai_") :],
        )
        for attr in attributes
        if attr.startswith("conc_bonsai_")
    ]

    attributes = dir(classifications.flowobject.datapackage)
    conc_flowobject = [
        (
            getattr(
                getattr(classifications, "flowobject"), "datapackage"
            ).__getattribute__(attr),
            attr[len("conc_bonsai_") :],
        )
        for attr in attributes
        if attr.startswith("conc_bonsai_")
    ]

    attributes = dir(classifications.flow.datapackage)
    conc_flow = [
        (
            getattr(getattr(classifications, "flow"), "datapackage").__getattribute__(
                attr
            ),
            attr[len("concpair_bonsai_") :],
        )
        for attr in attributes
        if attr.startswith("concpair_bonsai_")
    ]

    # Desired column order
    new_order = [
        "tree_bonsai_activitytype",
        "tree_bonsai_flowobject",
        "tree_other_activitytype",
        "tree_other_flowobject",
        "other_classification",
        "comment",
        "skos_uri",
        "prefixed_id",
    ]

    conc_merged = conc_flow[0][0]
    for c in conc_flow[1:]:
        conc_merged = pd.concat([conc_merged, c[0]], ignore_index=True)

    conc_merged = conc_merged[new_order]

    for c in conc_activitytype:
        c_transformed = restructure_dataframe(
            original_df=c[0], folder_name="activitytype", suffix="tree_" + c[1]
        )
        c_transformed = c_transformed[new_order]
        conc_merged = pd.concat([conc_merged, c_transformed], ignore_index=True)

    for c in conc_flowobject:
        c_transformed = restructure_dataframe(
            original_df=c[0], folder_name="flowobject", suffix="tree_" + c[1]
        )
        c_transformed = c_transformed[new_order]
        conc_merged = pd.concat([conc_merged, c_transformed], ignore_index=True)

    print(conc_merged)
    # Create the output directory if it doesn't exist
    os.makedirs(args.output_dir, exist_ok=True)

    output_path = f"{args.output_dir}/merged_concordance.csv"
    # Save each DataFrame in the result as a CSV file only if it has changed
    conc_merged.to_csv(output_path, index=False)


if __name__ == "__main__":
    main()



import csv
import os
import tempfile
from pathlib import Path

ROOT_PATH = Path(os.path.dirname(__file__))


def update_levels(file_path):
    def read_csv(file_path):
        with open(file_path, mode="r", encoding="utf-8-sig") as file:
            reader = csv.DictReader(file)
            data = [row for row in reader]
        return data, reader.fieldnames

    def write_csv(file_path, fieldnames, rows):
        with tempfile.NamedTemporaryFile(
            mode="w", delete=False, newline="", dir=os.path.dirname(file_path)
        ) as temp_file:
            writer = csv.DictWriter(temp_file, fieldnames=fieldnames)
            writer.writeheader()
            writer.writerows(rows)
            temp_file_path = temp_file.name
        os.replace(temp_file_path, file_path)

    def compute_level(row, data):
        # Ensure that the 'code' field exists in the row
        if "code" not in row:
            raise KeyError(f"Missing 'code' field in row: {row}")
        level = 0
        parent_code = row["parent_code"]
        visited_codes = set()
        while parent_code != "":
            if parent_code in visited_codes:
                # Circular reference detected
                raise ValueError(
                    f"Circular reference detected starting at code '{row}' in file '{file_path}'"
                )
            visited_codes.add(parent_code)
            level += 1
            parent_row = next(
                (item for item in data if item.get("code") == parent_code), None
            )
            if parent_row is None:
                raise ValueError(
                    f"Parent code '{parent_code}' for code '{row['code']}'."
                )

            parent_code = parent_row.get("parent_code", "")

        return level

    # Read the CSV file
    data, fieldnames = read_csv(file_path)

    # Ensure 'level' column is added to fieldnames
    if "level" not in fieldnames:
        fieldnames.append("level")

    # Compute levels for each row
    for row in data:
        try:
            row["level"] = compute_level(row, data)
        except ValueError as e:
            print(e)

    # Write the updated rows back to the same CSV file
    write_csv(file_path, fieldnames, data)


if __name__ == "__main__":
    directories = [
        ROOT_PATH.joinpath("data/flow/activitytype"),
        ROOT_PATH.joinpath("data/dataquality"),
        ROOT_PATH.joinpath("data/flow/flowobject"),
        ROOT_PATH.joinpath("data/location"),
        ROOT_PATH.joinpath("data/time"),
        ROOT_PATH.joinpath("data/uncertainty"),
        ROOT_PATH.joinpath("data/unit"),
    ]
    for d in directories:
        csv_files = [
            os.path.join(d, file) for file in os.listdir(d) if file.endswith(".csv")
        ]
        for file in csv_files:
            print(f"{file}")
            if "tree_" in str(file):
                update_levels(file_path=file)



import sys

from ._utils import create_conc, get_concordance
from .core import (
    activitytype,
    dataquality,
    flow,
    flowobject,
    location,
    time,
    uncertainty,
    unit,
)

# NOTE: Do not edit from here downward
# Create package version number from git tag
if sys.version_info[:2] >= (3, 8):
    from importlib.metadata import PackageNotFoundError, version
else:
    from importlib_metadata import PackageNotFoundError, version

# Change package version if project is renamed
try:
    dist_name = __name__
    __version__ = version(dist_name)
except PackageNotFoundError:  # pragma: no cover
    __version__ = "unknown"
finally:
    del version, PackageNotFoundError, dist_name, sys



# generate_excel.py
import os

import pandas as pd


def generate_excel_files():
    # Get the directory where the script is located
    script_dir = os.path.dirname(os.path.abspath(__file__))

    directories = [
        "flow/activitytype",
        # "dataquality",
        "flow/flowobject",
        "location",
        # "time",
        # "uncertainty",
        # "unit",
    ]

    # Define the output directory for Excel files within the Sphinx _static directory
    excel_dir = os.path.join(script_dir, "_static", "excel_files")

    # Create the output directory if it doesn't exist
    os.makedirs(excel_dir, exist_ok=True)

    # Convert all CSV files to Excel
    for d in directories:
        csv_dir = os.path.normpath(
            os.path.join(script_dir, "..", "src", "classifications", "data", d)
        )

        dict_with_dfs = {}

        for csv_file in os.listdir(csv_dir):
            if csv_file.endswith(".csv") and "conc_bonsai" in csv_file:
                # Read the CSV file
                csv_path = os.path.join(csv_dir, csv_file)
                df = pd.read_csv(csv_path, encoding="utf-8", sep=",", dtype="str")
                dict_with_dfs[f"{csv_file}"] = df
            if csv_file.endswith(".csv") and "tree_bonsai" in csv_file:
                # Read the CSV file
                csv_path = os.path.join(csv_dir, csv_file)
                df_merged = pd.read_csv(
                    csv_path, encoding="utf-8", sep=",", dtype="str"
                )

        matching_keys = [key for key in dict_with_dfs if "tree_bonsai" in key]
        if len(matching_keys) > 1:
            raise ValueError(
                f"Too many files with 'tree_bonsai' in filename inside {d}"
            )
        try:
            # df_merged = dict_with_dfs[matching_keys[0]]
            for k, df in dict_with_dfs.items():
                if k != "tree_bonsai":
                    column_to_merge = "tree_bonsai"

                    df = df.drop(columns=["comment", "prefixed_id", "skos_uri"])
                    df = (
                        df.groupby("tree_bonsai")[df.columns[1]]
                        .apply(list)
                        .reset_index()
                    )

                    df_merged = pd.merge(
                        df_merged,
                        df,
                        left_on="code",
                        right_on=str(column_to_merge),
                        how="left",
                    )
                    df_merged = df_merged.drop(columns=["tree_bonsai"])

        except (KeyError, IndexError):
            pass

        # Define the Excel file name and path
        if d.startswith("flow/"):
            d = d[len("flow/") :]
        else:
            pass
        excel_file = f"{d}_tree_bonsai.xlsx"
        excel_path = os.path.join(excel_dir, excel_file)
        # Save the data as an Excel file
        df_merged.to_excel(excel_path, index=False)


if __name__ == "__main__":
    generate_excel_files()



import csv
import os
import shutil
import tempfile
from logging import getLogger
from pathlib import Path

ROOT_PATH = Path(os.path.dirname(__file__))

logger = getLogger("root")

# Define a function to determine the comment based on the counts
def get_comment(source_count, target_count, s, t):
    if source_count == 1 and target_count == 1:
        return "one-to-one correspondence"
    elif source_count > 1 and target_count == 1:
        return "one-to-many correspondence"
    elif source_count == 1 and target_count > 1:
        return "many-to-one correspondence"
    elif source_count > 1 and target_count > 1:
        logger.info(
            f"'many-to-many' relation occured when mapping source code '{s}' to target code '{t}'. Make sure that you only map source codes to target codes each with the most disaggregated level!"
        )
        return "many-to-many correspondence"
    else:
        return ""


# Define a function to determine the skos ui based on the counts
def get_skos_uri(source_count, target_count):
    if source_count == 1 and target_count == 1:
        return "http://www.w3.org/2004/02/skos/core#exactMatch"
    elif source_count > 1 and target_count == 1:
        return "http://www.w3.org/2004/02/skos/core#closeMatch"
    elif source_count == 1 and target_count > 1:
        return "http://www.w3.org/2004/02/skos/core#closeMatch"
    elif source_count > 1 and target_count > 1:
        return "http://www.w3.org/2004/02/skos/core#relatedMatch"
    else:
        return ""


def add_mapping_comment(csv_file):

    # Read CSV file
    with open(csv_file, mode="r", newline="", encoding="utf-8") as file:
        reader = csv.DictReader(file)
        rows = list(reader)
        column_names = reader.fieldnames

    source_name = column_names[0]
    target_name = column_names[1]

    # Update the rows with the comments
    for row in rows:
        source_count = sum(1 for r in rows if r[source_name] == row[source_name])
        target_count = sum(1 for r in rows if r[target_name] == row[target_name])
        row["comment"] = get_comment(
            source_count, target_count, s=row[source_name], t=row[target_name]
        )
        row["skos_uri"] = get_skos_uri(source_count, target_count)

    # Write updated data to a temporary file
    with tempfile.NamedTemporaryFile(
        mode="w", delete=False, newline="", encoding="utf-8"
    ) as temp_file:
        with open(temp_file.name, mode="w", newline="", encoding="utf-8") as file:
            # Write header
            fieldnames = (
                rows[0].keys() if rows else []
            )  # Get fieldnames from the first row
            writer = csv.DictWriter(file, fieldnames=fieldnames, lineterminator="\n")
            writer.writeheader()

            # Write rows
            for row in rows:
                # Handle values with commas
                formatted_row = []
                for value in row.values():
                    try:
                        if isinstance(value, list):
                            formatted_row.append(",".join(value))
                        elif "," in value:
                            formatted_row.append(f'"{value}"')
                        else:
                            formatted_row.append(value)
                    except TypeError:
                        pass
                file.write(",".join(formatted_row) + "\n")

    # Replace the original file with the temporary one
    shutil.move(temp_file.name, csv_file)


# Function to combine the two columns for source and the two columns for target
def combine_columns(row, col1, col2):
    return f"{row[col1]}-{row[col2]}" if col1 and col2 in row else ""


def add_mapping_comment_concpair(csv_file, overwrite=False):
    # Read the CSV file
    with open(csv_file, mode="r", newline="", encoding="utf-8") as file:
        reader = csv.DictReader(file)
        rows = list(reader)
        column_names = reader.fieldnames

    if not column_names or len(column_names) < 4:
        raise ValueError(
            "The CSV file must have at least four columns: two for source and two for target."
        )

    # Check if the "comment" column already exists
    comment_exists = "comment" and "skos_uri" in column_names

    # If the comment column exists, decide whether to overwrite or not
    if comment_exists and not overwrite:
        logger.info(
            "The 'comment' column already exists. Skipping update as overwrite_comment=False."
        )
        return

    # Assumes first two columns are source and next two are target
    source_col1, source_col2 = column_names[0], column_names[1]
    target_col1, target_col2 = column_names[2], column_names[3]

    # Update the rows with the comments
    for row in rows:
        # Combine the values from the two source and two target columns
        source_combined = combine_columns(row, source_col1, source_col2)
        target_combined = combine_columns(row, target_col1, target_col2)

        # Calculate source and target counts for each combined pair
        source_count = sum(
            1
            for r in rows
            if combine_columns(r, source_col1, source_col2) == source_combined
        )
        target_count = sum(
            1
            for r in rows
            if combine_columns(r, target_col1, target_col2) == target_combined
        )

        # Add the comment based on the correspondence
        row["comment"] = get_comment(
            source_count, target_count, s=source_combined, t=target_combined
        )
        row["skos_uri"] = get_skos_uri(source_count, target_count)

    # Write updated data to a temporary file
    with tempfile.NamedTemporaryFile(
        mode="w", delete=False, newline="", encoding="utf-8"
    ) as temp_file:
        # Writing updated rows to temp file
        fieldnames = column_names + ["comment"] + ["skos_uri"]
        writer = csv.DictWriter(temp_file, fieldnames=fieldnames, lineterminator="\n")
        writer.writeheader()
        writer.writerows(rows)

    # Replace the original file with the updated temp file
    shutil.move(temp_file.name, csv_file)


if __name__ == "__main__":
    directories = [
        ROOT_PATH.joinpath("data/flow/activitytype"),
        ROOT_PATH.joinpath("data/dataquality"),
        ROOT_PATH.joinpath("data/flow/flowobject"),
        ROOT_PATH.joinpath("data/location"),
        ROOT_PATH.joinpath("data/time"),
        ROOT_PATH.joinpath("data/uncertainty"),
        ROOT_PATH.joinpath("data/unit"),
        ROOT_PATH.joinpath("data/flow"),
    ]
    for d in directories:
        conc_csv_files = [
            os.path.join(d, file)
            for file in os.listdir(d)
            if file.endswith(".csv") and file.startswith("conc_")
        ]
        for file in conc_csv_files:
            add_mapping_comment(file)

        concpair_csv_files = [
            os.path.join(d, file)
            for file in os.listdir(d)
            if file.endswith(".csv") and file.startswith("concpair_")
        ]
        for file in concpair_csv_files:
            add_mapping_comment_concpair(file)



import os
from pathlib import Path

import pandas as pd
import pytest

from classifications import _mapping_type

test_file = Path(os.path.dirname(__file__)).parent / "tests/data/conc_test.csv"


def test_mapping_type():
    _mapping_type.add_mapping_comment(test_file)
    df_result = pd.read_csv(test_file)
    assert df_result["comment"].to_list() == [
        "one-to-many correspondence",
        "one-to-many correspondence",
        "one-to-one correspondence",
        "many-to-one correspondence",
        "many-to-one correspondence",
        "one-to-many correspondence",
        "many-to-many correspondence",
        "many-to-one correspondence",
    ]



import os
from logging import getLogger
from pathlib import Path

import pandas

from ._mapping_type import get_comment, get_skos_uri

logger = getLogger("root")

ROOT_PATH = Path(os.path.dirname(__file__))

activitytype_path = "data/flow/activitytype/"
location_path = "data/location/"
dataquality_path = "data/dataquality/"
unit_path = "data/unit/"
uncertainty_path = "data/uncertainty/"
time_path = "data/time/"
flowobject_path = "data/flow/flowobject/"
flow_path = "data/flow/"

# Lookup function for pandas DataFrame
def lookup(self, keyword):
    """Filter the DataFrame based on the keyword in the "name" column"""
    filtered_df = self[self["name"].str.contains(keyword)]
    return filtered_df


def get_children(self, parent_code):
    """Get all children of a certain parent_code"""
    try:
        filtered_df = self[self["parent_code"] == parent_code]
    except KeyError as e:
        raise KeyError("Data table has no column 'parent_code'") from e
    return filtered_df


def create_conc(df_A, df_B, source="", target="", intermediate=""):
    """Create new concordance based on two other tables.

    Argument
    --------
    df_A : pandas.DataFrame
        concordance table A
        with mapping from "x" to "y"
    df_B : pandas.DataFrame
        concordance table B
        with mapping from "y" to "z"
    target : str
        header name that specifies "x"
    source : str
        header name that specifies "z"
    intermediate : str
        header name that specifies "y"

    Returns
    -------
    pandas.DataFrame
        concordance table with mapping form "x" to "z"
    """
    new_mapping = pandas.merge(df_A, df_B, on=intermediate)

    # Drop duplicate pairs of source and target
    new_mapping = new_mapping.drop_duplicates(subset=[source, target])

    # Calculate the counts of each source and target in the merged DataFrame
    source_counts = new_mapping[source].value_counts().to_dict()
    target_counts = new_mapping[target].value_counts().to_dict()
    # Apply the get_comment function to each row
    new_mapping["comment"] = new_mapping.apply(
        lambda row: get_comment(
            source_counts[row[source]],
            target_counts[row[target]],
            s=row[source],
            t=row[target],
        ),
        axis=1,
    )
    new_mapping["skos_uri"] = new_mapping.apply(
        lambda row: get_skos_uri(
            source_counts[row[source]], target_counts[row[target]]
        ),
        axis=1,
    )

    new_mapping = new_mapping[[source, target, "comment", "skos_uri"]]
    new_mapping = new_mapping.reset_index(drop=True)
    return new_mapping


# Subclass pandas DataFrame
class CustomDataFrame(pandas.DataFrame):
    lookup = lookup
    get_children = get_children


def get_concordance(from_classification, to_classification):
    """
    Get the concordance DataFrame based on the specified classifications.
    Parameters
    ----------
    from_classification: str
        The source classification name (e.g., "bonsai").
    to_classification: str
        The target classification name (e.g., "nace_rev2").
    Returns
    -------
    pd.DataFrame
        The concordance DataFrame if 1 file is found; otherwise, a dict of DataFrames.
    """
    # Construct the file name
    file_names = [
        f"conc_{from_classification}_{to_classification}.csv",
        f"conc_{to_classification}_{from_classification}.csv",
    ]
    file_paths = [
        activitytype_path,
        location_path,
        dataquality_path,
        unit_path,
        uncertainty_path,
        time_path,
        flowobject_path,
        flow_path,
    ]
    multiple_dfs = {}
    for f in file_paths:
        for n in file_names:
            file_path = ROOT_PATH.joinpath(f, n)
            try:
                # Read the concordance CSV into a DataFrame
                concordance_df = pandas.read_csv(file_path, dtype=str)
                multiple_dfs[f"{f}"] = concordance_df
                # return multiple_dfs
            except FileNotFoundError:
                pass
            except Exception as e:
                logger.error(f"Error reading concordance file: {e}")
                return None
    if len(multiple_dfs):
        return multiple_dfs[next(iter(multiple_dfs))]
    else:
        return multiple_dfs



import csv
import os
import shutil
import tempfile
import uuid
from pathlib import Path

ROOT_PATH = Path(os.path.dirname(__file__))


def add_unique_identifiers(csv_file):

    # Read CSV file
    with open(csv_file, mode="r", newline="", encoding="utf-8") as file:
        reader = csv.DictReader(file)
        data = list(reader)

    # Identify empty rows and add UUIDs
    for row in data:
        if not row.get("prefixed_id"):
            row["prefixed_id"] = str(uuid.uuid4())

    # Write updated data to a temporary file
    with tempfile.NamedTemporaryFile(
        mode="w", delete=False, newline="", encoding="utf-8"
    ) as temp_file:
        with open(temp_file.name, mode="w", newline="", encoding="utf-8") as file:
            # Write header
            fieldnames = (
                data[0].keys() if data else []
            )  # Get fieldnames from the first row
            writer = csv.DictWriter(file, fieldnames=fieldnames, lineterminator="\n")
            writer.writeheader()

            # Write rows
            for row in data:
                # Handle values with commas
                formatted_row = []
                for value in row.values():
                    try:
                        if isinstance(value, list):
                            formatted_row.append(",".join(value))
                        elif "," in value:
                            formatted_row.append(f'"{value}"')
                        else:
                            formatted_row.append(value)
                    except TypeError:
                        pass
                file.write(",".join(formatted_row) + "\n")

    # Replace the original file with the temporary one
    shutil.move(temp_file.name, csv_file)


if __name__ == "__main__":
    directories = [
        ROOT_PATH.joinpath("data/flow/activitytype"),
        ROOT_PATH.joinpath("data/dataquality"),
        ROOT_PATH.joinpath("data/flow/flowobject"),
        ROOT_PATH.joinpath("data/location"),
        ROOT_PATH.joinpath("data/time"),
        ROOT_PATH.joinpath("data/uncertainty"),
        ROOT_PATH.joinpath("data/unit"),
        ROOT_PATH.joinpath("data/flow"),
    ]
    for d in directories:
        csv_files = [
            os.path.join(d, file) for file in os.listdir(d) if file.endswith(".csv")
        ]
        for file in csv_files:
            if "resources.csv" not in str(file):
                add_unique_identifiers(file)
